; File: Simco-Net-generator-2.nlogo
;
; Gives a handy tool with a little interface to generate networks.
;
; 2013/2014/2015/2016/2017
;
; Fabian Adelt, Andreas Ihrig (TU Dortmund)
;
; Licence:
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Extensions:
;   table - helpers for tables
;   array - helpers for arrays
;   string - string handling
;   pathdir - directory handling
;   file - file handling
;   stats - handling of simulation data, including some basic statistic methods
;
; Includes:
;   variables.nls - contains all breed- and global-variables (<libs/variables.nls>)
;   helpers.nls - contains some helper functions (<libs/helpers.nls>)
;   helpers-visualization.nls - contains code for visual representation of the macro-layer (<libs/helpers-visualization.nls>)
;   helpers-input.nls - contains heler functions for reading input files (<libs/helpers-input.nls>)
;   helpers-output.nls - contains helper functions for writing output files, generating the respective file-names etc. (<libs/helpers-output.nls>)
;   helpers-file-handling.nls - contains  functions to handle files and folders (<libs/helpers-file-handling.nls>)
;   helpers-lists.nls - contains functions for basic list operations (<libs/helpers-lists.nls>)
;   interactions.nls - contains functions handling all interactions between <agents>, <technologies> and infrastructure elements (<nodes> and <edges>) (<libs/interactions.nls>)

extensions [table array string pathdir file stats]
__includes ["libs/variables.nls" "libs/helpers.nls" "libs/helpers-visualization.nls" "libs/helpers-input.nls"
            "libs/helpers-output.nls" "libs/helpers-file-handling.nls" "libs/helpers-lists.nls"  "libs/interactions.nls"]

; Breed: globals
; Add further globals
;
; Variables:
;   dir - directory to store output
;   number-of-dimensions - # of dimensions
;   number-of-nodes - # of nodes
;   number-of-edges - # of edges
;   number-of-agents - # of agents
;   number-of-technologies - # of technologies
globals [
  dir
  number-of-dimensions ; as user input for debugging
  number-of-nodes
  number-of-edges
  number-of-agents
  number-of-technologies

  changes-list
]




; Function: choose-directory
;
; Let the user choose a directory. Directory chooser opens at subdir "networks". Chosen directory will be saved at "dir".
;
to choose-directory
  set-current-directory (word pathdir:get-model "/networks")
  user-message "Please specify your networks directory."
  set dir user-directory
  ifelse (dir != false) [
    output-write "You selected: "
    output-print dir
  ]
  [
    output-write "You did not select any directory."
    set dir ""
  ]
end


; Function: append-to-changes-list
; Appends the given string and source to the changes-list
;
; Parameters:
;   item-type - string, indicating type of change
;   value-list - list, containing file-name to be memorized
;
; Returns:
;   list - changes list, supplemented by input elements
;
to append-to-changes-list [item-type value-list]
  set changes-list (append-to-list changes-list item-type value-list)
end


; Function: import-dimensions
; Import dimensions with names (maybe other values later)
;
to import-dimensions [dims-file]
  file-close
  set changes-list []
  let file dims-file

  if (dims-file = "") [
    set-current-directory (word pathdir:get-model pathdir:get-separator "networks" pathdir:get-separator "types")
    set file user-file
  ]

  if (file != false) [
    ; read in data from file
    file-open file
    let content ""
    if (file-at-end?) [print "at end"]
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]

    set names-of-dimensions []

    while [not file-at-end?] [
      set content file-read-line ; read in next line

      if (not empty? content) [
        ; read in data from content
        let data read-list-from-string content

        ifelse (length data = 1) [
          set number-of-dimensions (number-of-dimensions + 1)
          set names-of-dimensions lput (item 0 data) names-of-dimensions
          output-print (word "added dimension \"" (item 0 data) "\"")
        ][
        print "[dims import] ignoring incomplete line"
        ]
      ]
    ]

    ifelse (number-of-dimensions = 0) [
      user-message "no dimensions found, please choose a none empty file!"
    ][
    ; add file to changes-list
    append-to-changes-list "dims" (list get-file-name file)
    ]
  ]
  file-close
end

; Function: import-technologies
; Import technologies from file. The number of dimensions here defines them for the hole network.
;
to import-technology-types [techs-file]
  let number-one-dimensional 5
  file-close
  let file techs-file

  if (techs-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set file user-file
  ]

  if (file != false) [
    ; read in data from file
    file-open file
    let content ""
    if (file-at-end?) [print "at end"]
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]

    while [not file-at-end?] [
      set content file-read-line ; read in next line

      if (not empty? content) [
        ; read in data from content
        let data read-list-from-string content

        ifelse (((length data) - number-one-dimensional) = number-of-dimensions) [
          create-technology-types 1 [
;            set technology-type-id (number-of-technologies + 1)
            set technology-type-name (item 0 data)
            set technology-type-purchase-price (read-from-string (item 1 data))
            set technology-type-lifetime-mean (read-from-string (item 2 data))
            set technology-type-lifetime-deviation (read-from-string (item 3 data))
            set technology-type-duration-factor (read-from-string (item 4 data))

            ;technology-factor
            let i 0
            set technology-type-factor []
            repeat (length data - number-one-dimensional) [
              set technology-type-factor lput (read-from-string (item (number-one-dimensional + i) data)) technology-type-factor
              set i (i + 1)
            ]
          ]

        output-print (word "imported technology \"" (item 0 data) "\"")

        set number-of-technologies (number-of-technologies + 1)
        ][
        user-message (word "[techs import] data for technology-type " item 0 data "not complete, one line has to many or to few entries")
        user-message (word "With " number-of-dimensions " dimensions the number of entries should be " (number-of-dimensions * 1 + number-one-dimensional) ", but it is "length data)
        clear-turtles
        stop
        ]
      ]
    ]

    ; add file to changes-list
    append-to-changes-list "techs" (list get-file-name file)
  ]
  file-close
end


; Function: names-of-agent-types
; Reports the names of all agent types
;
to-report names-of-agent-types
  set agent-types []
  foreach sort agents [
    if not (member? ([agent-type] of ?) agent-types) [set agent-types lput [agent-type] of ? agent-types]
  ]
  report agent-types
end

; Function: filter-available-technologies
; Filters only available technologies
;
to-report filter-available-technologies [available-techs techs]
  let filtered-techs []
  foreach techs [
    ifelse (is-number? (position ? available-techs)) [
      set filtered-techs lput ? filtered-techs
    ][
    print (word "ignoring unavailable technology: " ? who myself self)
    ]
  ]
  report filtered-techs
end

;Function: import-node-type
; Imports a node type and creates a node-type-object
;

to-report import-node-type [my-nodes-file]
  let number-one-dimensional 4  ; number of single-dimensional variables
  file-close
  let file my-nodes-file
  let my-type-name ""

  if (my-nodes-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set file user-file
  ]

  if (file != false) [
    ; read in data from file
    file-open file
    let content ""
    if (file-at-end?) [print "at end"]
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]
    ;print (word "content: " content)
    if (not file-at-end?) [set content file-read-line] ; read in intresting line
                                                       ;print (word "content: " content)
    set-default-shape nodes "circle"
    ; read in data from content
    let data read-list-from-string content

    ; check if there is data for each dimensions (read from technologies file)
    ifelse (length data = (number-one-dimensional + number-of-dimensions * 11)) [
      create-node-types 1 [
        set node-type-name item 0 data ;my-nodes-file
        set my-type-name node-type-name
        set node-type-lifetime-mean read-from-string (item 1 data)
        set node-type-lifetime-deviation read-from-string (item 2 data)
        ;technologies
        let available-techs names-of-technology-types
        let techs (item 3 data)
        set node-type-right-of-use (filter-available-technologies available-techs techs)
        ;local vars for import of limits
        let my-upper-limit 0
        let my-lower-limit 0
        let my-mean 0
        let my-dev 0
        ;node-limits-short-time
        set node-type-limits-short-time []
        let j 0
        repeat number-of-dimensions [
          set my-upper-limit (read-from-string (item (number-one-dimensional + 0 * number-of-dimensions + j) data))
          set my-lower-limit (read-from-string (item (number-one-dimensional + 1 * number-of-dimensions + j) data))
          set node-type-limits-short-time lput (list my-lower-limit my-upper-limit) node-type-limits-short-time
          set j j + 1
        ]
        ;node-limits-long-time
        set node-type-limits-long-time []
        set j 0
        repeat number-of-dimensions [
          set my-upper-limit (read-from-string (item (number-one-dimensional + 2 * number-of-dimensions + j) data))
          set my-lower-limit (read-from-string (item (number-one-dimensional + 3 * number-of-dimensions + j) data))
          set node-type-limits-long-time lput (list my-lower-limit my-upper-limit) node-type-limits-long-time
          set j j + 1
        ]
        ;node-costs-benefits-nodes
        set node-type-costs-benefits-nodes-mean []
        set node-type-costs-benefits-nodes-deviation []
        set j 0
        repeat number-of-dimensions [
          set node-type-costs-benefits-nodes-mean lput (read-from-string (item (number-one-dimensional + 4 * number-of-dimensions + j) data)) node-type-costs-benefits-nodes-mean
          set node-type-costs-benefits-nodes-deviation lput (read-from-string (item (number-one-dimensional + 5 * number-of-dimensions + j) data)) node-type-costs-benefits-nodes-deviation
          set j j + 1
        ]
        ;node-costs-benefits-agents
        set node-type-costs-benefits-agents-mean []
        set node-type-costs-benefits-agents-deviation []
        set j 0
        repeat number-of-dimensions [
          set node-type-costs-benefits-agents-mean lput (read-from-string (item (number-one-dimensional + 6 * number-of-dimensions + j) data)) node-type-costs-benefits-agents-mean
          set node-type-costs-benefits-agents-deviation lput (read-from-string (item (number-one-dimensional + 7 * number-of-dimensions + j) data)) node-type-costs-benefits-agents-deviation
          set j j + 1
        ]
        ;node-costs-benefits-external
        set node-type-costs-benefits-external-mean []
        set node-type-costs-benefits-external-deviation []
        set j 0
        repeat number-of-dimensions [
          set node-type-costs-benefits-external-mean lput (read-from-string (item (number-one-dimensional + 8 * number-of-dimensions + j) data)) node-type-costs-benefits-external-mean
          set node-type-costs-benefits-external-deviation lput (read-from-string (item (number-one-dimensional + 9 * number-of-dimensions + j) data)) node-type-costs-benefits-external-deviation
          set j j + 1
        ]
        ;node-costs-benefits-technology-dependencies
        set node-type-costs-benefits-technology-dependencies []
        let technology-dependency false
        set j 0
        repeat number-of-dimensions [
          set technology-dependency read-from-string (item (number-one-dimensional + 10 * number-of-dimensions + j) data)
          if(not is-boolean? technology-dependency) [set technology-dependency false]
          set node-type-costs-benefits-technology-dependencies lput technology-dependency node-type-costs-benefits-technology-dependencies
          set j j + 1
        ]
      ]
    ]
    [
      user-message (word "[nodes import] data for node-type " item 0 data " not complete, number of entries doesn't fit!" )
      user-message (word "With " number-of-dimensions " dimensions the number of entries should be " (number-of-dimensions * 11 + number-one-dimensional) ", but it is "length data)
    ]
  ]
  file-close

  report my-type-name
end


; Function: create-node-of-type
; Creates a node of given type, using a node-type-object to derive the neccessary information.
;
to create-node-of-type [my-node-type]
  ;print my-node-type
  let my-type one-of node-types with [node-type-name = my-node-type]

  ifelse(my-type != NOBODY) [
    ;create a new node and set its variables according to the given type.
    create-nodes 1 [
      set node-name my-node-type
      set node-id (count nodes)
      set node-lifetime random-normal [node-type-lifetime-mean] of my-type [node-type-lifetime-deviation] of my-type
      set node-right-of-use [node-type-right-of-use] of my-type
      set node-limits-short-time [node-type-limits-short-time] of my-type
      set node-limits-long-time [node-type-limits-long-time] of my-type
      set node-costs-benefits-nodes (map random-normal [node-type-costs-benefits-nodes-mean] of my-type [node-type-costs-benefits-nodes-deviation] of my-type)
      set node-costs-benefits-agents (map random-normal [node-type-costs-benefits-agents-mean] of my-type [node-type-costs-benefits-agents-deviation] of my-type)
      set node-costs-benefits-external (map random-normal [node-type-costs-benefits-external-mean] of my-type [node-type-costs-benefits-external-deviation] of my-type)
      set node-costs-benefits-technology-dependencies ([node-type-costs-benefits-technology-dependencies] of my-type)

      set node-agents turtle-set nobody
      ;;initialize node-pay-offs/degree with the same number of elements as there are in node-costs-benefits-nodes
      set node-pay-offs-short-time createList node-costs-benefits-nodes 0
      set node-pay-offs-long-time createList node-costs-benefits-nodes 0
      set node-degree-short-time createList node-costs-benefits-nodes 0
      set node-degree-long-time createList node-costs-benefits-nodes 0

      set node-overcrowded? false
      set node-history table:make

      ; for bimodal networks add -1, -2 and -c to node-name to specify edge-creation
      if bimodal? [
        if last node-name != "1" AND last node-name != "2" AND last node-name != "c" [user-message "No bimodal-type found. For bimodal networks add -1, -2 and -c to node-name to specify edge-creation"]
        set node-type-bimodal last node-name
        set node-name substring node-name 0 (length node-name - 2)
      ]
    ]
  ][
    user-message "[nodes create] type not found"
  ]

  set number-of-nodes count nodes
end

; Function: create-edge-of-type
; Instantiates an edge (fromt the calling node to the sink given, and sets its parameters according to the type given.
;
; Parameters:
;   my-edge-sink - <node> that shall be reached by the <edge> instantiated within this function.
;   my-edge-type - the <edge-type> that shall be used to instantiate the <edge>
;
to create-edge-of-type [my-edge-sink my-edge-type]
  ;print my-edge-type
  let my-type one-of edge-types with [edge-type-name = my-edge-type]

  ifelse(my-type != NOBODY) [
    create-edge-to my-edge-sink [
      set edge-name my-edge-type
      set number-of-edges count edges
      set edge-id number-of-edges
      set edge-lifetime random-normal [edge-type-lifetime-mean] of my-type [edge-type-lifetime-deviation] of my-type
      set edge-duration random-normal [edge-type-duration-mean] of my-type [edge-type-duration-deviation] of my-type
      set edge-right-of-use [edge-type-right-of-use] of my-type

      set edge-limits-short-time [edge-type-limits-short-time] of my-type
      set edge-limits-long-time [edge-type-limits-long-time] of my-type

      ;edge-costs-benefits
      set edge-costs-benefits-edges (map random-normal [edge-type-costs-benefits-edges-mean] of my-type [edge-type-costs-benefits-edges-deviation] of my-type)
      set edge-costs-benefits-agents (map random-normal [edge-type-costs-benefits-agents-mean] of my-type [edge-type-costs-benefits-agents-deviation] of my-type)
      set edge-costs-benefits-external (map random-normal [edge-type-costs-benefits-external-mean] of my-type [edge-type-costs-benefits-external-deviation] of my-type)
      set edge-costs-benefits-technology-dependencies ([edge-type-costs-benefits-technology-dependencies] of my-type)

      ;initialize edge-pay-offs/degree with the same number of elements as there are in edge-costs-benefits-edges
      set edge-pay-offs-short-time createList edge-costs-benefits-edges 0
      set edge-pay-offs-long-time createList edge-costs-benefits-edges 0

      set edge-degree-short-time createList edge-costs-benefits-edges 0
      set edge-degree-long-time createList edge-costs-benefits-edges 0

      ;initialize edge-agents-list
      set edge-agents turtle-set nobody
      set edge-history table:make

      set edge-overcrowded? false
    ]
  ][
    user-message "[edges create] type not found"
  ]
end

; Function: import-nodes-of-type
; Imports a node type and generates a number of nodes queried from user
;
to import-nodes-of-type [my-nodes-file my-nodes-number]
  if (my-nodes-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set my-nodes-file user-file
  ]
  let my-type-name import-node-type my-nodes-file
  let number my-nodes-number
  set number round read-from-string user-input "How many nodes do you want?"
  ifelse (number != false AND is-number? number) [
    repeat number [
      create-node-of-type my-type-name
    ]
  ][
  user-message "[nodes import] no number given!"
  ]
  ; add file and number to changes-list
  ;print my-nodes-file
  append-to-changes-list "nodes" (list (get-file-name my-nodes-file) number)

end

; Function: import-edge-type
; Imports a edge-type and creates a edge-type-object
;
to-report import-edge-type [my-edge-file]
  let number-one-dimensional 6 ; number of single-dimensional variables
  file-close
  let my-type-name ""

  if (my-edge-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set my-edge-file user-file
  ]

  if (my-edge-file != false) [
    ; read in data from file
    file-open my-edge-file
    let content ""
    if (file-at-end?) [print "at end"]
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]
    if (not file-at-end?) [set content file-read-line] ; read in intresting line
    ifelse (not empty? content) [
      ; read in data from content
      let data read-list-from-string content

      ; check if there is data for each dimensions (read from technologies file)
      ifelse (length data = (number-one-dimensional + number-of-dimensions * 11)) [
        ; get names of all technologies
        let available-techs names-of-technology-types
        create-edge-types 1 [
          set edge-type-name item 0 data ;get-file-name my-edge-file
          set my-type-name edge-type-name
          set edge-type-lifetime-mean (read-from-string (item 1 data ))
          set edge-type-lifetime-deviation (read-from-string (item 2 data))
          set edge-type-duration-mean (read-from-string (item 3 data))
          set edge-type-duration-deviation (read-from-string (item 4 data))
          let techs (item 5 data)
          ;let techs read-from-string (word "[\"" string:rex-replace-all " " (item 4 data) "\" \"" "\"]")
          set edge-type-right-of-use (filter-available-technologies available-techs techs)

          ;local vars for import
          let my-upper-limit 0
          let my-lower-limit 0
          let my-mean 0
          let my-dev 0

          ;edge-limits-short-time
          set edge-type-limits-short-time []
          let j 0
          repeat number-of-dimensions [
            set my-upper-limit (read-from-string (item (number-one-dimensional + 0 * number-of-dimensions + j) data))
            set my-lower-limit (read-from-string (item (number-one-dimensional + 1 * number-of-dimensions + j) data))
            set edge-type-limits-short-time lput (list my-lower-limit my-upper-limit) edge-type-limits-short-time
            set j j + 1
          ]
          ;edge-limits-long-time
          set edge-type-limits-long-time []
          set j 0
          repeat number-of-dimensions [
            set my-upper-limit (read-from-string (item (number-one-dimensional + 2 * number-of-dimensions + j) data))
            set my-lower-limit (read-from-string (item (number-one-dimensional + 3 * number-of-dimensions + j) data))
            set edge-type-limits-long-time lput (list my-lower-limit my-upper-limit) edge-type-limits-long-time
            set j j + 1
          ]

          ;edge-costs-benefits-nodes
          set edge-type-costs-benefits-edges-mean []
          set edge-type-costs-benefits-edges-deviation []
          set j 0
          repeat number-of-dimensions [
            set edge-type-costs-benefits-edges-mean lput (read-from-string (item (number-one-dimensional + 4 * number-of-dimensions + j) data)) edge-type-costs-benefits-edges-mean
            set edge-type-costs-benefits-edges-deviation lput  (read-from-string (item (number-one-dimensional + 5 * number-of-dimensions + j) data)) edge-type-costs-benefits-edges-deviation
            set j j + 1
          ]
          ;edge-costs-benefits-agents
          set edge-type-costs-benefits-agents-mean []
          set edge-type-costs-benefits-agents-deviation []
          set j 0
          repeat number-of-dimensions [
            set edge-type-costs-benefits-agents-mean lput  (read-from-string (item (number-one-dimensional + 6 * number-of-dimensions + j) data)) edge-type-costs-benefits-agents-mean
            set edge-type-costs-benefits-agents-deviation lput (read-from-string (item (number-one-dimensional + 7 * number-of-dimensions + j) data)) edge-type-costs-benefits-agents-deviation
            set j j + 1
          ]
          ;edge-costs-benefits-external
          set edge-type-costs-benefits-external-mean []
          set edge-type-costs-benefits-external-deviation []
          set j 0
          repeat number-of-dimensions [
            set edge-type-costs-benefits-external-mean lput (read-from-string (item (number-one-dimensional + 8 * number-of-dimensions + j) data)) edge-type-costs-benefits-external-mean
            set edge-type-costs-benefits-external-deviation lput   (read-from-string (item (number-one-dimensional + 9 * number-of-dimensions + j) data))edge-type-costs-benefits-external-deviation
            set j j + 1
          ]
          ;edge-costs-benefits-technology-dependencies
          set edge-type-costs-benefits-technology-dependencies []
          let technology-dependency false
          set j 0
          repeat number-of-dimensions [
            set technology-dependency read-from-string (item (number-one-dimensional + 10 * number-of-dimensions + j) data)
            if(not is-boolean? technology-dependency) [set technology-dependency false]
            set edge-type-costs-benefits-technology-dependencies lput technology-dependency edge-type-costs-benefits-technology-dependencies
            set j j + 1
          ]
        ]
      ][
        user-message (word "[edges import] data for edge-type " item 0 data " not complete, number of entries doesn't fit!")
        user-message (word "With " number-of-dimensions " dimensions the number of entries should be " (number-of-dimensions * 11 + number-one-dimensional) ", but it is "length data)
      ]
    ][
    ;todo: content not sufficient.
    ]
  ]
  file-close

  report my-type-name
end

; Function: import-edges-of-type
; Imports an edge type and generates a number of edges queried from user, connecting nodes more or less randomly
;
to import-edges-of-type [my-edges-file my-edges-number]

  if (my-edges-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set my-edges-file user-file
  ]

  if (my-edges-file != false) [
    let number my-edges-number
    if (my-edges-number = false) [
      set number round read-from-string user-input "How many edges do you want?"
    ]
    ifelse (number != false AND is-number? number) [
      ; read in edge-type
      let my-type-name import-edge-type my-edges-file
      ;create as many edges of this type as required
      repeat number [
        ;set from-node
        let from-node 0
        ; use unconnected nodes first to avoid unconnected nodes in the resulting graph.
        ifelse (count nodes with [count my-out-links = 0] > 0) [
          set from-node [node-id] of one-of nodes with [count my-out-links = 0]
        ][
          ; avoid to generate a clustered graph.
          ifelse (count nodes with [count my-out-links = 1] > 0) [
            set from-node [node-id] of one-of nodes with [count my-out-links = 1]
          ][
            set from-node [node-id] of one-of nodes with [count my-out-links < (count nodes - 1 )]
          ]
        ]

        ; set to-node
        let to-node 0
        ; if there are any unreachable nodes (not equal to the from-node chosen above), use on of them as to-node, otherwise a random node
        ifelse (count nodes with [count my-in-links = 0 AND node-id !=  from-node] > 0)[
          set to-node [node-id] of one-of nodes with [count my-in-links = 0 AND node-id != from-node]
        ][
        let candidates nodes with [
              (node-id != from-node) AND
              (not in-link-neighbor?  one-of nodes with [node-id = from-node]) AND
                count my-in-links = 1
        ]
        ifelse ( count candidates > 0) [
          set to-node [node-id] of one-of candidates
          ][
            ;from- and to-node must be different and from-node must not be connected to to-node in advance.
            set to-node [node-id] of one-of nodes with [
              (node-id != from-node) AND
              (not in-link-neighbor?  one-of nodes with [node-id = from-node] )
            ]
          ]
        ]
        let k 1
        if create-two-directed-edges? [ set k 2]
        repeat k [
          ask get-node from-node [
            create-edge-of-type get-node to-node my-type-name ;get-file-name my-edges-file
            let tmp to-node
            set to-node from-node
            set from-node tmp
          ]
        ]

      ]

      ; add file and number to changes-list.
      append-to-changes-list "edges" (list (get-file-name my-edges-file) number)

      ;; lay it out so links are not overlapping
    ;  repeat 500 [ layout ]
      ;; leave space around the edges
      ask nodes [ setxy 0.95 * xcor 0.95 * ycor ]
    ][
    user-message "[edges import] no number given!"
    ]
  ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  Bimodal Network Edge Creation  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to import-edges-of-type-bimodal [my-edges-file my-edges-number]

  if (my-edges-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set my-edges-file user-file
  ]

  if (my-edges-file != false) [
    let number my-edges-number
    if (my-edges-number = false) [
      set number round read-from-string user-input "How many edges do you want?"
    ]
    ifelse (number != false AND is-number? number) [
      ; read in edge-type
      let my-type-name import-edge-type my-edges-file
      ;create as many edges of this type as required



     let i 1
     loop [
      if i = 3 [stop]

      let number-bimodal 0
      ifelse i = 1 [
        set number-bimodal round (number * bimodal-edge-allocation-factor)
      ]
      [
        set number-bimodal (number - round (number * bimodal-edge-allocation-factor))
      ]



      repeat number-bimodal [
        ;set from-node
        let from-node 0
        ; use unconnected nodes first to avoid unconnected nodes in the resulting graph.
;Idea: let from-node always be of the connecting node type and then repeat two times to create % of Edges to either of the other node types
; from-node in second stage needs to be distinguished by out-links to nodes with node-type-bimodal = "2" = 0
        ifelse i = 1 [
           ifelse (count nodes with [count my-out-links = 0 AND node-type-bimodal = "c"] > 0) [
             set from-node [node-id] of one-of nodes with [count my-out-links = 0 AND node-type-bimodal = "c"]
;the connecting node must be chosen here
           ][
             ; avoid to generate a clustered graph.
             ifelse (count nodes with [count my-out-links = 1 AND node-type-bimodal = "c"] > 0) [
               set from-node [node-id] of one-of nodes with [count my-out-links = 1 AND node-type-bimodal = "c"]
             ][
               set from-node [node-id] of one-of nodes with [count my-out-links < (count nodes - 1 ) AND node-type-bimodal = "c"]
             ]
           ]
        ]
        [
           ifelse (count nodes with [count out-link-neighbors with [node-type-bimodal = "2"] = 0 AND node-type-bimodal = "c"] > 0) [
             set from-node [node-id] of one-of nodes with [count out-link-neighbors with [node-type-bimodal = "2"] = 0 AND node-type-bimodal = "c"]
;the connecting node must be chosen here
           ][
             ; avoid to generate a clustered graph.
             ifelse (count nodes with [count out-link-neighbors with [node-type-bimodal = "2"] = 1 AND node-type-bimodal = "c"] > 0) [
               set from-node [node-id] of one-of nodes with [count out-link-neighbors with [node-type-bimodal = "2"] = 1 AND node-type-bimodal = "c"]
             ][
               set from-node [node-id] of one-of nodes with [count out-link-neighbors with [node-type-bimodal = "2"] < (count nodes - 1 ) AND node-type-bimodal = "c"]
             ]
           ]
        ]

        ;the receiving nodes must be chosen here, but two times to ensure the two types only get edges to the connecting node type
        let to-node 0
        ifelse i = 1 [
;;;;;;;;;;;; set to-node type 1
           ; if there are any unreachable nodes (not equal to the from-node chosen above), use on of them as to-node, otherwise a random node
           ;here the unreachable node of the node type to be connected must be specified
           ifelse (count nodes with [count my-in-links = 0 AND node-id != from-node AND node-type-bimodal = "1"] > 0)[
             set to-node [node-id] of one-of nodes with [count my-in-links = 0 AND node-id != from-node AND node-type-bimodal = "1"]
           ][
           let candidates nodes with [
                 (node-id != from-node) AND
                 (not in-link-neighbor?  one-of nodes with [node-id = from-node]) AND
                  count my-in-links = 1 AND node-type-bimodal = "1"
           ]
           ifelse ( count candidates > 0) [
             set to-node [node-id] of one-of candidates
           ][
            ;from- and to-node must be different and from-node must not be connected to to-node in advance.
               set to-node [node-id] of one-of nodes with [
                 (node-id != from-node) AND
                 (not in-link-neighbor?  one-of nodes with [node-id = from-node] AND node-type-bimodal = "1")
               ]
            ]
           ]
        ]

        [
;;;;;;;;;;;; set to-node type 2
           ; if there are any unreachable nodes (not equal to the from-node chosen above), use on of them as to-node, otherwise a random node
           ;here the unreachable node of the node type to be connected must be specified
           ifelse (count nodes with [count my-in-links = 0 AND node-id !=  from-node AND node-type-bimodal = "2"] > 0)[
             set to-node [node-id] of one-of nodes with [count my-in-links = 0 AND node-id != from-node AND node-type-bimodal = "2"]
           ][
           let candidates nodes with [
                 (node-id != from-node) AND
                 (not in-link-neighbor?  one-of nodes with [node-id = from-node]) AND
                  count my-in-links = 1 AND node-type-bimodal = "2"
           ]
           ifelse ( count candidates > 0) [
             set to-node [node-id] of one-of candidates
           ][
            ;from- and to-node must be different and from-node must not be connected to to-node in advance.
              set to-node [node-id] of one-of nodes with [
                (node-id != from-node) AND
                (not in-link-neighbor?  one-of nodes with [node-id = from-node] AND node-type-bimodal = "2")
             ]
           ]
           ]
        ]

        let k 1
        if create-two-directed-edges? [ set k 2]
            repeat k [
            ask get-node from-node [
              create-edge-of-type get-node to-node my-type-name ;get-file-name my-edges-file
              let tmp to-node
              set to-node from-node
              set from-node tmp
            ]
            ]


; here repeat number ends
      ]

;here loop ends
     set i i + 1
     ]

      ; add file and number to changes-list.
      append-to-changes-list "edges" (list (get-file-name my-edges-file) number)

      ;; lay it out so links are not overlapping
    ;  repeat 500 [ layout ]
      ;; leave space around the edges
      ask nodes [ setxy 0.95 * xcor 0.95 * ycor ]
    ][
    user-message "[edges import] no number given!"
    ]
  ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  END  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;






to import-p-calculations [my-p-calculations-file]
  file-close
  let file my-p-calculations-file

  if (my-p-calculations-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set file user-file
  ]

  ifelse (file != false) [
    set p-calculations table:make

    ; read in data from file
    file-open file
    let content ""
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]

    while [not file-at-end?] [
      let data read-list-from-string file-read-line

      ifelse (length data < 2) [
        print "skipped p calculations row cause of too few elements"
      ][
      table:put p-calculations (length table:keys p-calculations + 1) (list (item 0 data) (item 1 data))
      ]
    ]

    ; add file to changes-list
    append-to-changes-list "p-calculations" (list (get-file-name file))

    file-close

    ; save number of aims
    set number-of-seu-aims (length table:keys p-calculations)
    output-print (word "import " number-of-seu-aims " p-calculations from " file)
  ][
  user-message "[p-calculations import] file has no data line!"
  ]
end

; Function: import-agents-of-type
; Imports a agent type and generates a number of agents queried from user
to import-agents-of-type [my-agents-file my-agents-number]
  let number-one-dimensional 2 ; number of single-dimensional variables
  file-close
  let file my-agents-file

  if (my-agents-file = "") [
    set-current-directory (word pathdir:get-model "/networks/types")
    set file user-file
  ]

  ; check if we have to create the u-values and destinations tables
  carefully [
    let keys table:keys u-default
  ][
  set u-default table:make
  ]
  carefully [
    let keys table:keys destinations-once-default
  ][
    set destinations-once-default table:make
  ]
  carefully [
    let keys table:keys destination-conditions-once-default
  ][
  set destination-conditions-once-default table:make
  ]
  carefully [
    let keys table:keys destinations-repetitive-default
  ][
  set destinations-repetitive-default table:make
  ]
  carefully [
    let keys table:keys destination-conditions-repetitive-default
  ][
  set destination-conditions-repetitive-default table:make
  ]
  carefully [
    let keys table:keys technology-switch-conditions
  ][
  set technology-switch-conditions table:make
  ]


  if (file != false) [
    ; read in data from file
    file-open file
    let content ""
    if (file-at-end?) [print "at end"]
    if (not file-at-end?) [
      set content file-read-line ; overread first line, this is the header
      set content ""
    ]
    ;print (word "content: " content)
    if (not file-at-end?) [set content file-read-line] ; read in intresting line

    ifelse (not empty? content) [
      let number my-agents-number
      if (my-agents-number = false) [
        set number round read-from-string user-input "How many agents do you want?"
      ]

      ifelse (number != false AND is-number? number) [
        ; read in data from content
        let data read-list-from-string content

        ; check if there is data for each dimensions (read from technologies file)
        ifelse (length data >= (number-one-dimensional + number-of-dimensions * 4 + number-of-seu-aims + 1)) [
          ; get names of all technologies
          let available-techs names-of-technology-types
          let i number-of-agents
          let j 0
          repeat number [
            ;create a new agent
            create-agents 1 [
              set agent-id i
              set agent-type (item 0 data)

              ;technologies
              let techs (item 1 data)
              set agent-technologies-available (filter-available-technologies available-techs techs)
              set agent-technology-in-use one-of agent-technologies-available

              ;take a random location ;todo: add starting-nodes(?)
              let start-nodes [];(list 1 5 7 )
              ifelse not empty? start-nodes [
                set agent-location one-of nodes with [member? node-id start-nodes]
              ][
              set agent-location one-of nodes
              ]
              ask agent-location [
                set node-agents (turtle-set node-agents myself)
              ]
              set agent-destination -1 ;no destination choosen, yet

              set agent-edge-delay 0 ;agents wait on nodes, so delay must be zero

                                     ;local vars for import
              let my-upper-limit 0
              let my-lower-limit 0

              ;agent-limits-short-time
              set agent-limits-short-time []
              set j 0
              repeat number-of-dimensions [
                set my-upper-limit (read-from-string (item (number-one-dimensional + 0 * number-of-dimensions + j) data))
                set my-lower-limit (read-from-string (item (number-one-dimensional + 1 * number-of-dimensions + j) data))
                set agent-limits-short-time lput (list my-lower-limit my-upper-limit) agent-limits-short-time
                set j j + 1
              ]

              ;agent-limits-long-time
              set agent-limits-long-time []
              set j 0
              repeat number-of-dimensions [
                set my-upper-limit (read-from-string (item (number-one-dimensional + 2 * number-of-dimensions + j) data))
                set my-lower-limit (read-from-string (item (number-one-dimensional + 3 * number-of-dimensions + j) data))
                set agent-limits-long-time lput (list my-lower-limit my-upper-limit) agent-limits-long-time
                set j j + 1
              ]

              ;agent-pay-offs
              set agent-pay-offs-short-time createList number-of-dimensions 0
              set agent-pay-offs-long-time createList number-of-dimensions 0
              set agent-pay-offs createList number-of-dimensions 0

              set label-color black
            ]

            ;increase counter
            set i (i + 1)
          ]

          ;read this type's seu parameters
          let u-values []
          set j 0
          repeat number-of-seu-aims [
            set u-values lput (read-from-string (item (number-one-dimensional + 4 * number-of-dimensions + j) data)) u-values
            set j j + 1
          ]
          table:put u-default item 0 data u-values
          ;if (length (table:get (item 0 data) u-default) != number-of-seu-aims) [
          ;  output-print (word "number of aims don't fit to number of seu values; could cause import error in SimCo")
          ;]
          table:put destinations-once-default item 0 data (item (number-one-dimensional + 4 * number-of-dimensions + number-of-seu-aims) data)
          table:put destination-conditions-once-default item 0 data (item (number-one-dimensional + 4 * number-of-dimensions + number-of-seu-aims + 1 ) data)
          table:put destinations-repetitive-default item 0 data (item (number-one-dimensional + 4 * number-of-dimensions + number-of-seu-aims + 2 ) data)
          table:put destination-conditions-repetitive-default item 0 data (item (number-one-dimensional + 4 * number-of-dimensions + number-of-seu-aims + 3) data)
          table:put technology-switch-conditions item 0 data (item (number-one-dimensional + 4 * number-of-dimensions + number-of-seu-aims + 4) data)

          ; add file and number to changes-list
          append-to-changes-list "agents" (list (get-file-name file) number)

          set number-of-agents (number-of-agents + number)
          layout

          output-print (word number " agents from " file " created")
        ][
          user-message (word "[agents import] data for agent-type " item 0 data " not complete, number of entries doesn't fit!")
          user-message (word "With " number-of-dimensions " dimensions the number of entries should be " (number-of-dimensions * 4 + number-one-dimensional + number-of-seu-aims) ", but it is "length data)
        ]
      ][
      user-message "[agents import] no number given!"
      ]
    ][
    user-message "[agents import] file has no data line!"
    ]
    file-close
  ]
end

; Function: save-as-master
; Saves the changes-list as master file which could be load later instead of doing all steps one by one
;
to save-as-master
  choose-directory
  set-directory
  file-close
  ifelse (name-of-configuration != "") [
    ;create file name
    let master-file-name create-master-file-path word name-of-configuration "-master.txt"
    print master-file-name
    ;clear and generate matser-file
    if file-exists? master-file-name [
      let date string:rex-replace-all ":" date-and-time "-"
      set date (word substring date 16 27 "-" substring date 0 8 "-" substring date 13 15)
      set master-file-name (word substring master-file-name 0 (length master-file-name - 4 ) "-" date ".txt")
    ]
    file-open master-file-name

    carefully [
      ; gui changes list
      let gui-changes-list []

      ; add settings from gui
      set gui-changes-list append-to-list gui-changes-list "limits" (list short-time-limit long-time-multiplicator)
      set gui-changes-list append-to-list gui-changes-list "num-seu-aims" (list number-of-seu-aims)
      set gui-changes-list append-to-list gui-changes-list "logging?" (list logging?)
      set gui-changes-list append-to-list gui-changes-list "order-by-edge-id?" (list order-by-edge-id?)
      set gui-changes-list append-to-list gui-changes-list "create-two-directed-edges?" (list create-two-directed-edges?)
      set gui-changes-list append-to-list gui-changes-list "with-calculations-file?" (list with-calculations-file?)
      set gui-changes-list append-to-list gui-changes-list "as-zip-file?" (list as-zip-file?)
      set gui-changes-list append-to-list gui-changes-list "node-condition-once?" (list node-condition-once?)
      set gui-changes-list append-to-list gui-changes-list "node-condition-repetitive?" (list node-condition-repetitive?)

      ; save gui settings
      let i 0
      repeat (length gui-changes-list) [
        file-print (item i gui-changes-list)
        set i (i + 1)
      ]

      ; save other changes
      set i 0
      repeat (length changes-list) [
        file-print (item i changes-list)
        set i (i + 1)
      ]

      ;close the file
      file-close
      output-print "master saved"
    ][
    user-message error-message
    ]
  ][
  user-message "Please specify the configuration name."
  ]
end

; Function: read-from-master
; Reads a master file and generates a netwok based on its content.
;
to read-from-master
  file-close
  clear-all

  ifelse (pathdir:isDirectory? (word pathdir:get-model "/networks/masters")) [
    set-current-directory (word pathdir:get-model "/networks/masters")
  ][
  set-current-directory (word pathdir:get-model "/networks")
  ]

  let master-file user-file

  ifelse (master-file = false) [
    user-message "You did not select any master file."
  ][
  let file-name get-file-name master-file
  set name-of-configuration substring file-name 0 (length file-name - 4)

  let path get-path master-file
  ;retrieve network-path
  set dir get-parent-path path
  ;set working directory.
  set-directory
  if (master-file != false) [
    ; read in data from file
    file-open master-file
    let content []
    if (file-at-end?) [print "at end"]

    ; load data from master and check if everything is okay
    let my-short-time-limit -1
    let my-long-time-multiplicator -1
    let my-dims-file ""
    let my-techs-file ""
    let my-item-additions []

    while [not file-at-end?] [
      set content read-from-string (word "[" file-read-line "]")

      if (length content > 0) [

        if ((item 0 content) = "limits") [
          ifelse (length content = 3) [
            set my-short-time-limit read-from-string (item 1 content)
            set my-long-time-multiplicator read-from-string (item 2 content)
          ][
          user-message "import of time limits failed"
          stop
          ]
        ]

        if ((item 0 content) = "dims") [
          ifelse (length content = 2) [
            set my-dims-file create-type-file-path (item 1 content)
            if (not file-exists? my-dims-file) [
              user-message (word "dimensions file " my-dims-file " doesn't exist")
              stop
            ]
          ][
          user-message "import of dims failed"
          stop
          ]
        ]

        if ((item 0 content) = "techs") [
          ifelse (length content = 2) [
            set my-techs-file create-type-file-path (item 1 content)
            if (not file-exists? my-techs-file) [
              user-message (word "technologies file " my-techs-file " doesn't exist")
              stop
            ]
          ][
          user-message (word "import of techs failed")
          stop
          ]
        ]

        if ((item 0 content) = "nodes") [
          ifelse (length content = 3) [
            let my-nodes-file create-type-file-path (item 1 content)
            let my-nodes-number read-from-string (item 2 content)
            if ((not file-exists? my-nodes-file)) [
              user-message (word "nodes file  " my-nodes-file " doesn't exist")
              stop
            ]
            if (my-nodes-number = false) [
              user-message word "Import of number of nodes number failed for file " my-nodes-file
              stop
            ]
            if (my-nodes-number = 0) [
                user-message (word "Caution: No nodes will be created based on " my-nodes-file " as their number is set to 0.")
            ]
            set my-item-additions lput (list "nodes" my-nodes-file my-nodes-number) my-item-additions
          ][
          user-message "import of nodes failed"
          stop
          ]
        ]

        if ((item 0 content) = "edges") [
          ifelse (length content = 3) [
            let my-edges-file create-type-file-path (item 1 content)
            let my-edges-number read-from-string (item 2 content)
            if ((not file-exists? my-edges-file)) [
              user-message (word "edges file " my-edges-file " doesn't exist")
              stop
            ]
            if (my-edges-number = false) [
              user-message "edges number failed to import or zero"
              stop
            ]
            if (my-edges-number = 0) [
              user-message (word "Caution: No edges will be created based on " my-edges-file " as their number is set to 0.")
            ]
            set my-item-additions lput (list "edges" my-edges-file my-edges-number) my-item-additions
          ][
          user-message "import of edges failed"
          stop
          ]
        ]

        if ((item 0 content) = "p-calculations") [
          ifelse (length content = 2) [
            let my-p-calculations-file create-type-file-path (item 1 content)
            if ((not file-exists? my-p-calculations-file)) [
              user-message (word "p-calculations file " my-p-calculations-file " doesn't exist")
              stop
            ]
            set my-item-additions lput (list "p-calculations" my-p-calculations-file) my-item-additions
          ][
          user-message "import of p-calculations failed"
          stop
          ]
        ]

        if ((item 0 content) = "agents") [
          ifelse (length content = 3) [
            let my-agents-file create-type-file-path (item 1 content)
            let my-agents-number read-from-string (item 2 content)
            if ((not file-exists? my-agents-file)) [
              user-message (word "agents file " my-agents-file " doesn't exist")
              stop
            ]
            if (my-agents-number = false) [
              user-message "agents number failed to import or zero"
              stop
            ]
            if (my-agents-number = 0) [
              user-message (word "Caution: No agents will be created based on " my-agents-file " as their number is set to 0.")
            ]
            set my-item-additions lput (list "agents" my-agents-file my-agents-number) my-item-additions
          ][
          user-message "import of agents failed"
          stop
          ]
        ]

        if ((item 0 content) = "num-seu-aims") [
          ifelse (length content = 2) [
            set number-of-seu-aims read-from-string (item 1 content)
          ][
          user-message "import of number-of-seu-aims failed"
          stop
          ]
        ]

        if ((item 0 content) = "logging?") [
          ifelse (length content = 2) [
            set logging? read-from-string (item 1 content)
          ][
          user-message "import of logging? failed"
          stop
          ]
        ]

        if ((item 0 content) = "order-by-edge-id?") [
          ifelse (length content = 2) [
            set order-by-edge-id? read-from-string (item 1 content)
          ][
          user-message "import of order-by-edge-id? failed"
          stop
          ]
        ]

        if ((item 0 content) = "create-two-directed-edges?") [
          ifelse (length content = 2) [
            set create-two-directed-edges? read-from-string (item 1 content)
          ][
          user-message "import of create-two-directed-edges? failed"
          stop
          ]
        ]

        if ((item 0 content) = "with-calculations-file?") [
          ifelse (length content = 2) [
            set with-calculations-file? read-from-string (item 1 content)
          ][
          user-message "import of with-calculations-file? failed"
          stop
          ]
        ]

        if ((item 0 content) = "as-zip-file?") [
          ifelse (length content = 2) [
            set as-zip-file? read-from-string (item 1 content)
          ][
            user-message "import of as-zip-file? failed"
            stop
          ]
        ]
        if ((item 0 content) = "node-condition-once?") [
          ifelse (length content = 2) [
            set node-condition-once? read-from-string (item 1 content)
          ][
            user-message "import of node-condition-once? failed"
            stop
          ]
        ]
        if ((item 0 content) = "node-condition-repetitive?") [
          ifelse (length content = 2) [
            set node-condition-repetitive? read-from-string (item 1 content)
          ][
            user-message "import of node-condition-repetitive? failed"
            stop
          ]
        ]
      ]
    ]

    ; check if everything was set
    if (short-time-limit = false or short-time-limit < 0) [
      user-message "import short-time-limit failed (maybe not set)"
      stop
    ]
    if (long-time-multiplicator = false or long-time-multiplicator < 0) [
      user-message "import long-time-multiplicator failed (maybe not set)"
      stop
    ]
    if (my-dims-file = "") [
      user-message "import of dims file failed (maybe not set)"
      stop
    ]
    if (my-techs-file = "") [
      user-message "import of techs file failed (maybe not set)"
      stop
    ]

    ; everything is okay, so now we could set the globals and start import of data
    set short-time-limit my-short-time-limit
    set long-time-multiplicator my-long-time-multiplicator
    import-dimensions my-dims-file
    import-technology-types my-techs-file
    foreach (my-item-additions) [
      if ((item 0 ?) = "nodes") [
        let my-type-name import-node-type (item 1 ? )
        repeat  (item 2 ?) [create-node-of-type my-type-name]
      ]
      if ((item 0 ?) = "edges") [
        import-edges-of-type (item 1 ?) (item 2 ?)
      ]
      if ((item 0 ?) = "p-calculations") [
        import-p-calculations (item 1 ?)
      ]
      if ((item 0 ?) = "agents") [
        import-agents-of-type (item 1 ?) (item 2 ?)
      ]
    ]

    file-close
  ]
  ]
end

; Function: save-network
; Writes out the network into 6 files. All file names start with the network name.
;
to save-network
  choose-directory
  set-directory
  file-close
  ifelse (name-of-configuration != "") [
    ;create file names
    let setup-file-name word name-of-configuration "-setup.txt"
    let nodes-file-name word name-of-configuration "-nodes.txt"
    let edges-file-name word name-of-configuration "-edges.txt"
    let agents-file-name word name-of-configuration "-agents.txt"
    let technologies-file-name word name-of-configuration "-technologies.txt"
    let globals-file-name word name-of-configuration "-globals.txt"
    let seu-values-file-name word name-of-configuration "-seu-values.txt"
    let p-calculations-file-name word name-of-configuration "-p-calculations.txt"

    ;create directory and set apropriet if wanted as zip file
    if (as-zip-file?) [
      if(not pathdir:isDirectory? name-of-configuration) [ pathdir:create name-of-configuration ]
      set-current-directory (word pathdir:get-current pathdir:get-separator name-of-configuration)

      set nodes-file-name "nodes.txt"
      set edges-file-name "edges.txt"
      set agents-file-name "agents.txt"
      set technologies-file-name "technologies.txt"
      set globals-file-name "globals.txt"
      set seu-values-file-name "seu-values.txt"
      set p-calculations-file-name "p-calculations.txt"
    ]

    ;clear and generate setup-file (if not as zip file)
    if (not as-zip-file?) [
      if file-exists? setup-file-name [
        let date string:rex-replace-all ":" date-and-time "-"
        set date (word substring date 16 27 "-" substring date 0 8 "-" substring date 13 15)
        set setup-file-name (word substring setup-file-name 0 (length setup-file-name - 4 ) "-" date ".txt")

        file-close
        file-delete setup-file-name
      ]
      file-open setup-file-name
      ;write names of files
      file-print (word "\"nodes-file\"" " \"" nodes-file-name "\"")
      file-print (word "\"edges-file\"" " \"" edges-file-name "\"")
      file-print (word "\"agents-file\"" " \"" agents-file-name "\"")
      file-print (word "\"technologies-file\"" " \"" technologies-file-name "\"")
      file-print (word "\"globals-file\"" " \"" globals-file-name "\"")
      file-print (word "\"seu-values-file\"" " \"" seu-values-file-name "\"")
      if with-calculations-file? [file-print (word "\"p-calculations-file\"" " \"" p-calculations-file-name "\"")]
      ;close the file
      file-close
    ]

    ;clear and generate globals-file
    let long-time-limit short-time-limit * long-time-multiplicator
    if file-exists? globals-file-name [
      file-close
      file-delete globals-file-name
    ]
    file-open globals-file-name
    ;write global datas
    file-print (word "\"number-dimensions\" " number-of-dimensions )
    file-print (word "\"names-of-dimensions\" " "\"" (string-list-to-string names-of-dimensions) "\" ")
    file-print (word "\"short-time-limit\" " short-time-limit)
    file-print (word "\"long-time-limit\" " long-time-limit)
    file-print (word "\"node-condition-once\" " node-condition-once?)
    file-print (word "\"node-condition-repetitive\" " node-condition-repetitive?)
    ;close the file
    file-close

    ;clear and generate nodes-file
    if file-exists? nodes-file-name [
      file-close
      file-delete nodes-file-name
    ]
    file-open nodes-file-name
    ;data on the line is in this order:
    ;  id lifetime right-of-use [limits-short-time] [limits-long-time] [pay-offs-nodes] [pay-offs-agents] [pay-offs-external]
    foreach sort-on [node-id] nodes [
      ask ? [
        file-print (word
          "\"" node-name "\" "
          node-id " "
          node-lifetime " "
          "\"" (string-list-to-string node-right-of-use) "\" "
        "\"" (list-to-string node-limits-short-time " ") "\" "
        "\"" (list-to-string node-limits-long-time " ") "\" "
        "\"" (list-to-string node-costs-benefits-nodes " ") "\" "
        "\"" (list-to-string node-costs-benefits-agents " ") "\" "
        "\"" (list-to-string node-costs-benefits-external " ") "\" "
        "\"" (list-to-string node-costs-benefits-technology-dependencies " ") "\"")
        ;add EOL
        ;file-type "\r\n"
      ]
    ]
    ;close the file
    file-close

    ;clear and generate edges-file
    if file-exists? edges-file-name [
      file-close
      file-delete edges-file-name
    ]
    file-open edges-file-name

    ;data on the line is in this order:
    ;  from-node to-node edge-lifetime edge-right-of-use [edge-limits-short-time] [edge-limits-long-time] edge-duration [edge-pay-offs-edges] [edge-pay-offs-agents] [edge-pay-offs-external]

    ; todo: check if neccessary
;    foreach sort-by [sort-edges ?1 ?2 order-by-edge-id?] edges [
 ;     ask ? [
 ask edges [
        file-print (word
          "\"" edge-name "\" "
          [node-id] of (get-from-node self) " "
          [node-id] of (get-to-node self) " "
        edge-lifetime " "
        "\"" (string-list-to-string edge-right-of-use) "\" "
        "\"" (list-to-string edge-limits-short-time " ") "\" "
        "\"" (list-to-string edge-limits-long-time " ") "\" "
        edge-duration " "
        "\"" (list-to-string edge-costs-benefits-edges " ") "\" "
        "\"" (list-to-string edge-costs-benefits-agents " ") "\" "
        "\"" (list-to-string edge-costs-benefits-external " ") "\" "
        "\"" (list-to-string edge-costs-benefits-technology-dependencies " ") "\"")
      ]
  ;  ]
    ;close the file
    file-close

    ;clear and generate technologies-file
    if file-exists? technologies-file-name [
      file-close
      file-delete technologies-file-name
    ]
    file-open technologies-file-name
    ;data on the line is in this order:
    ;  technology-id, technology-name, technology-purchase-price, technology-lifetime, technology-speed-factor technology-factor
    foreach sort-on [technology-type-name] technology-types [
      ask ? [
        file-print (word
          "\"" technology-type-name "\" "
          technology-type-purchase-price " "
          technology-type-lifetime-mean " "
          technology-type-lifetime-deviation " "
          technology-type-duration-factor " "
          "\"" (list-to-string technology-type-factor " ") "\"")
      ]
    ]
    ;close the file
    file-close

    ;clear and generate agents-file
    if file-exists? agents-file-name [
      file-close
      file-delete agents-file-name
    ]
    file-open agents-file-name
    ;data on the line is in this order:
    ;  agent-id agent-location agent-destination agent-edge-delay agent-limits-short-time agent-limits-long-time agent-pay-offs agent-technologies-available agent-technology-in-use
    foreach sort-on [agent-id] agents [
      ask ? [
        file-print (word
          agent-id " "
          [node-id] of agent-location " "
        agent-destination " "
        agent-edge-delay " "
        "\"" (list-to-string agent-limits-short-time " ") "\" "
        "\"" (list-to-string agent-limits-long-time " ") "\" "
        "\"" (list-to-string agent-pay-offs " ") "\" "
        "\"" (string-list-to-string agent-technologies-available) "\" "
        "\"" agent-technology-in-use "\" "
        "\"" agent-type "\"")
      ]
    ]
    ;close the file
    file-close

    ;clear and generate seu-values-file
    if file-exists? seu-values-file-name [
      file-close
      file-delete seu-values-file-name
    ]
    file-open seu-values-file-name
    set agent-types names-of-agent-types
    foreach agent-types [
      ; u values and agent-destinations.
      let u-values table:get u-default ?
      let destinations-once table:get destinations-once-default ?
      let destination-conditions-once table:get destination-conditions-once-default ?
      let destinations-repetitive table:get destinations-repetitive-default ?
      let destination-conditions-repetitive table:get destination-conditions-repetitive-default ?
      let technology-switch-condition table:get technology-switch-conditions ?
      ; write these values.
      file-print (word "\"" ? "\" \"" u-values "\" \"" destinations-once "\" \"" (string:rex-replace-all "\\\"" (destination-conditions-once) "\\\\\"") "\" \"" destinations-repetitive"\" \"" (string:rex-replace-all "\\\"" (destination-conditions-repetitive) "\\\\\"")"\" \""  (string:rex-replace-all "\\\"" (technology-switch-condition) "\\\\\"") "\"")
    ]
    ;close the file
    file-close

    if with-calculations-file? [
      ;clear and generate seu-values-file
      if file-exists? p-calculations-file-name [
        file-close
        file-delete p-calculations-file-name
      ]
      file-open p-calculations-file-name
      let calcs []
      foreach table:keys p-calculations [
        set calcs table:get p-calculations ?
        file-print (word "\"" (string:rex-replace-all "\\\"" (item 0 calcs) "\\\\\"") "\" \"" (string:rex-replace-all "\\\"" (item 1 calcs) "\\\\\"") "\"")
      ]
      ;close the file
      file-close
    ]

    if (as-zip-file?) [
      set-directory
      let configuration-to-pack name-of-configuration
      if (file-exists? (word name-of-configuration ".snf")) [
           let date string:rex-replace-all ":" date-and-time "-"
           set date (word substring date 16 27 "-" substring date 0 8 "-" substring date 13 15)
           set name-of-configuration (word name-of-configuration "-" date)

      ]

      file:zip-file-pack-dir configuration-to-pack (word name-of-configuration ".snf")

      ;remove files
      set-current-directory (word pathdir:get-current pathdir:get-separator configuration-to-pack)
      file-delete nodes-file-name
      file-delete edges-file-name
      file-delete agents-file-name
      file-delete technologies-file-name
      file-delete globals-file-name
      file-delete seu-values-file-name
      if with-calculations-file? [file-delete p-calculations-file-name]

      ;remove directory
      set-directory
      pathdir:delete configuration-to-pack
    ]

    output-print "configuration saved"
    save-network-description
  ]
  [
    user-message "Please give a name for this configuration"
  ]

end

; Function: save-network-description
; Saves the characteristics of the types used to create the scenario. This can be used as a starting point for a scenario-description by including verbal fragments to the respective parts.
; Uses wikimedia style formats, such that the output-file can be inserted to any wiki page.
;
to save-network-description
  ; create file, name consists of configuration-name, sufficed by "-description.txt"
  ; if old file exists, delete it and start from scratch.
  let description-file-name (word name-of-configuration "-description.txt")
  if file-exists? description-file-name [
    file-close
    file-delete description-file-name
  ]
  file-open description-file-name

  ;set some standard texts to be printed with the respective output-element
  let limit-help "Each limit contains two values: The first value represents a lower limit, while the second value represents an upper limit. The value \"false\" indicates that the respective value is not set. In other words: No limit is specified."
  let cost-benefit-help "Each cost-benefit-entry contains two values. The first is mean value, the second deviation, used as input to a normal distribution when building real nodes/edges based on the types described here."

  print-heading-of-order 2 "General notes"
  file-print "The following page lists the details of a SimCo scenario. It is automatically generated based on the scenario's data and eventually supplemented by additional comments."

  ;the file shall be subdivided based on the dimensions specified
  foreach names-of-dimensions [
    let my-dimension ?
    let number-of-dimension get-dimension ?
    ; type-names and type-values collect data from all types available, such that they can be printed nicely formatted in the end.
    let type-names []
    let type-values []

    foreach sort node-types [
      set type-names lput (word [node-type-name] of ? "-node") type-names
    ]
    foreach sort edge-types [
      set type-names lput (word [edge-type-name] of ? "-edge") type-names
    ]

    ;
    print-heading-of-order 2 my-dimension
    ; retrieve and print short-time-limits of nodes and edges
    set type-values []
    print-heading-of-order 3 "short-time-limits"
    file-print limit-help
    foreach sort node-types [
      set type-values lput (check-dontcare item number-of-dimension [node-type-limits-short-time] of ?) type-values
    ]
    foreach sort edge-types [
      set type-values lput (check-dontcare item number-of-dimension [edge-type-limits-short-time] of ?) type-values
    ]
    ;print the tabular
     wikitable-print (list type-names type-values)
    ;
    ; retrieve and print long-time-limits of nodes and edges
    set type-values []
    print-heading-of-order 3 "long-time-limits"
    file-print limit-help
    foreach sort node-types [
      set type-values lput (check-dontcare item number-of-dimension [node-type-limits-long-time] of ?) type-values
    ]
    foreach sort edge-types [
      set type-values lput (check-dontcare item number-of-dimension [edge-type-limits-long-time] of ?) type-values
    ]
    ;print the tabular
    wikitable-print (list type-names type-values)
    ;
    ; retrieve and print cost-benefits of nodes and edges for themselves
    set type-values []
    print-heading-of-order 3 "cost-benefits for myself"
    file-print cost-benefit-help
    foreach sort node-types [
      set type-values lput (list
        (item number-of-dimension [node-type-costs-benefits-nodes-mean] of ?)
        (item number-of-dimension [node-type-costs-benefits-nodes-deviation] of ?)
      ) type-values
    ]
    foreach sort edge-types [
      set type-values lput (list
        (item number-of-dimension [edge-type-costs-benefits-edges-mean] of ?)
        (item number-of-dimension [edge-type-costs-benefits-edges-deviation] of ?)
      ) type-values
    ]
    ;print the tabular
     wikitable-print (list type-names type-values)
    ;
    ; retrieve and print cost-benefits of nodes and edges for agents
    set type-values []
    print-heading-of-order 3 "cost-benefits for agents"
    file-print cost-benefit-help
    foreach sort node-types [
      set type-values lput (list
        (item number-of-dimension [node-type-costs-benefits-agents-mean] of ?)
        (item number-of-dimension [node-type-costs-benefits-agents-deviation] of ?)
      ) type-values
    ]
    foreach sort edge-types [
      set type-values lput (list
        (item number-of-dimension [edge-type-costs-benefits-agents-mean] of ?)
        (item number-of-dimension [edge-type-costs-benefits-agents-deviation] of ?)
      ) type-values
    ]
    ;print the tabular
     wikitable-print (list type-names type-values)
     ; retrieve and print cost-benefits of nodes and edges for external/environment
    set type-values []
    print-heading-of-order 3 "cost-benefits for external/enviriment"
    file-print cost-benefit-help
    foreach sort node-types [
      set type-values lput (list
        (item number-of-dimension [node-type-costs-benefits-external-mean] of ?)
        (item number-of-dimension [node-type-costs-benefits-external-deviation] of ?)
      ) type-values
    ]
    foreach sort edge-types [
      set type-values lput (list
        (item number-of-dimension [edge-type-costs-benefits-external-mean] of ?)
        (item number-of-dimension [edge-type-costs-benefits-external-deviation] of ?)
      ) type-values
    ]
    ;print the tabular
     wikitable-print (list type-names type-values)
  ]
  file-close
end

; Function: print-heading-of-order
; Prints a heading of order n depending on the selected wiki style
; Parameter:
;   n - integer, order of the heading
;   message -  string,  name of the heading itself
to print-heading-of-order [n message]
  debug 4 "print-header-of-order"
  ifelse (redminewiki-style?) [
    file-print ("")
    file-print (word "h" n ". " message)
    file-print ("")
  ][
    let mediawiki-heading ""
    repeat n [
      set mediawiki-heading (word mediawiki-heading "=")
    ]
    file-print (word mediawiki-heading message mediawiki-heading)
  ]
end

; Function: wikitable-print
; Prints a tabular depending on the selected wiki style.

; Parameter:
;   list-of-lines - nested list, where each sub-list will be printed as a separate line and thus shall contain the same number of tabular cells
;
to wikitable-print [list-of-lines]
  debug 4 "wikitable-print"
  ifelse (redminewiki-style?) [
    redminewiki-table-print list-of-lines
  ][
    mediawiki-table-print list-of-lines
  ]
end


; Function: set-directory
; Sets the working directory to the one given via interface. If no directory given, a warning is issued.
; Should be used in generator before handling files for reading or writing.
;
to set-directory
  if (not (is-string? dir)) [
    set dir ""
  ]

  ifelse (dir != "") [
    set-current-directory dir
  ][
  user-message "Please specify the directory in the interface."
  ]

end



; Function: create-type-file-path
; Returns the full path to a file in the types directory
;
; Parameters:
;   file-name - String of file-name
; Returns:
;   full path
;
to-report create-type-file-path [file-name]
  let separator pathdir:get-separator
  report (word dir  "types" separator file-name)
end


; Function: create-master-file-path
; Returns the full path to a file in the masters' directory
;
; Parameters:
;   file-name - String of file-name
; Returns:
;   full path
;
to-report create-master-file-path [file-name]
  let separator pathdir:get-separator
  ;to be sure that the subdirectory "masters exists, create it. No action is taken if it already exists.
  pathdir:create "masters"

  report (word dir "masters" separator file-name)
end
@#$#@#$#@
GRAPHICS-WINDOW
840
200
1085
454
16
16
6.76
1
10
1
1
1
0
1
1
1
-16
16
-16
16
0
0
0
ticks
30.0

OUTPUT
10
10
825
160
11

BUTTON
390
510
550
555
save network
save-network\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
840
105
1070
176
name-of-configuration
DiscGo-realistic-middle-tenth
1
0
String

TEXTBOX
390
165
560
183
global configurations
15
0.0
1

SWITCH
580
270
805
303
create-two-directed-edges?
create-two-directed-edges?
0
1
-1000

SWITCH
580
190
805
223
logging?
logging?
1
1
-1000

BUTTON
10
290
185
335
add nodes of type
import-nodes-of-type \"\" 0
NIL
1
T
OBSERVER
NIL
N
NIL
NIL
1

MONITOR
200
290
375
335
# of nodes
number-of-nodes
17
1
11

BUTTON
840
10
950
55
reset
clear-all\nset-current-directory (word pathdir:get-model \"/networks/\")
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
965
10
1070
55
reload
__reload
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
390
190
565
250
short-time-limit
144
1
0
Number

INPUTBOX
390
250
565
310
long-time-multiplicator
30
1
0
Number

MONITOR
200
340
375
385
# of edges
number-of-edges
17
1
11

BUTTON
10
340
185
385
add edges of type
ifelse count nodes > 0 \n[ifelse bimodal? \n[import-edges-of-type-bimodal \"\" false]\n[import-edges-of-type \"\" false]\n]\n[user-message \"Please create some nodes first.\"]
NIL
1
T
OBSERVER
NIL
E
NIL
NIL
1

BUTTON
10
240
185
285
import technologies
import-technology-types \"\"
NIL
1
T
OBSERVER
NIL
T
NIL
NIL
1

MONITOR
200
240
375
285
# of technologies
number-of-technologies
17
1
11

MONITOR
200
190
375
235
# of dimensions
number-of-dimensions
17
1
11

MONITOR
390
310
565
355
long-time-limit
short-time-limit * long-time-multiplicator
17
1
11

BUTTON
10
440
185
485
add agents of type
import-agents-of-type \"\" false
NIL
1
T
OBSERVER
NIL
A
NIL
NIL
1

MONITOR
200
440
375
485
# of agents
number-of-agents
17
1
11

TEXTBOX
200
170
350
188
network configurations\n
12
0.0
1

TEXTBOX
10
170
160
188
modify network
12
0.0
1

BUTTON
10
190
185
235
import dimensions
import-dimensions \"\"
NIL
1
T
OBSERVER
NIL
D
NIL
NIL
1

BUTTON
200
510
375
555
save as master
save-as-master
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
10
510
185
555
import from master
if set-random-seed? [random-seed random 1000]\nread-from-master
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
850
460
930
505
Dead ends
count nodes with [count my-out-links = 0]
17
1
11

MONITOR
945
460
1070
505
unreachable nodes
count nodes with [count my-in-links = 0]
17
1
11

SWITCH
580
230
805
263
order-by-edge-id?
order-by-edge-id?
0
1
-1000

SWITCH
840
65
1070
98
as-zip-file?
as-zip-file?
0
1
-1000

TEXTBOX
580
165
730
183
other properties
15
0.0
1

SWITCH
580
310
805
343
with-calculations-file?
with-calculations-file?
0
1
-1000

BUTTON
10
390
185
435
import p calculations
import-p-calculations \"\"
NIL
1
T
OBSERVER
NIL
P
NIL
NIL
1

MONITOR
200
390
375
435
# of aims
number-of-seu-aims
17
1
11

INPUTBOX
580
345
732
405
number-of-seu-aims
8
1
0
Number

SWITCH
575
470
692
503
debugging?
debugging?
1
1
-1000

SWITCH
745
470
848
503
out?
out?
1
1
-1000

CHOOSER
575
510
713
555
debug-level
debug-level
0 1 2 3 4
4

CHOOSER
710
510
848
555
out-level
out-level
"SEU" "Setup"
0

INPUTBOX
745
405
830
465
out-who
0
1
0
Number

SWITCH
395
390
567
423
node-condition-once?
node-condition-once?
1
1
-1000

SWITCH
395
425
592
458
node-condition-repetitive?
node-condition-repetitive?
0
1
-1000

SWITCH
865
520
992
553
save-history?
save-history?
1
1
-1000

SWITCH
400
470
545
503
redminewiki-style?
redminewiki-style?
0
1
-1000

BUTTON
10
565
132
598
Set seed-value 1
let my-random 15896\nrandom-seed my-random\nprint (word \"set random-seed to \" my-random)\nset set-random-seed? false
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
10
600
135
633
Set seed-value 2
let my-random 1985864792\nrandom-seed my-random\nprint (word \"set random-seed to \" my-random)\nset set-random-seed? false
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
575
560
732
593
bimodal?
bimodal?
1
1
-1000

INPUTBOX
710
560
875
620
bimodal-edge-allocation-factor
0.375
1
0
Number

SWITCH
140
565
292
598
set-random-seed?
set-random-seed?
1
1
-1000

@#$#@#$#@
## SimCo Net Generator

This tool generates networks with technologies, nodes, edges and agents under some conditions. The conditions are defined per user interface.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
